/*
 * Copyright (C) 2001-2011 OSS Nokalva.  All rights reserved.
 *
 */

/*
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC. AND MAY BE USED
 * ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC. THIS FILE MAY NOT BE
 * DISTRIBUTED.
 */

/*
 * This example encodes/decodes/prints MYEXAMPLE Abstract Syntax. The
 * program illustrates how to assign values to the following types:
 *
 * NumericString,
 * OCTET STRING
 * SET OF,
 * SEQUENCE,
 * VisibleString,
 * INTEGER,
 * SET,
 * CHOICE,
 * BOOLEAN,
 * OBJECT IDENTIFIER,
 * GeneralizedTime.

/* In the main() routine you can see how to set up values to be encoded,
 * how to print the message that is ready to be encoded, how to encode the
 * message, how to print the encoded data, how to decode the encoded data,
 * how to print the decoded data, how to access the fields of the decoded data.
 */



#include <stdio.h>
#include <string.h>
#include "csample.h"  /* compiler-generated header file */

/**************************************************************/
/* arbritary values for octet strings to be used in encoding: */
/**************************************************************/
static unsigned char memNameOctStr[] = {
       0x52, 0x65, 0x61, 0x6C, 0x41, 0x64, 0x64, 0x72, 0x65, 0x73, 
       0x73, 0x52, 0x65, 0x61, 0x6C, 0x41, 0x64, 0x64, 0x72, 0x65,
       0x73, 0x73, 0x52, 0x65, 0x61, 0x6C, 0x41, 0x64, 0x64, 0x72
};

static char returnMsg1[32] = "Transaction 1 successful.";

static char returnMsg2[32] = "Transaction 2 unsuccessful.";

static unsigned char addrOctStr[] = {
       0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39,
       0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39,
       0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39
};

static unsigned char commtOctStr[] = {
       0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A,
       0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A,
       0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A
};

static unsigned char actionOctStr[] = {
       0x73, 0x74, 0x6F, 0x70, 0x2E
};

static unsigned char fldNmOctStr[] = {
       0x61, 0x62, 0x63, 0x64, 0x65, 0x61, 0x62, 0x63, 0x64, 0x65,
       0x61, 0x62, 0x63, 0x64, 0x65
}; /* ASCII ==> "abcdeabcdeabcde" */

/* proto-type */
void myDecodedDataPrinter(OssGlobal *, Msg1 *);

int main()
{
    OssBuf encodedData;  /* buffer to hold encoded data (length & value) */
    Msg1   myMsg;        /* compiler-generated structure for encoding    */
    Msg1   *DecodedDataPtr = NULL; /* pointer to reference decoded data  */
    int    myPduNum = 0; /* integer to store compiler-generated PDU tag  */
    OssGlobal w, *world = &w; /* OSS global API environment variable     */
    char *buf;                /* pointer to save address of encoded data */
    int retcode;              /* return code */
    struct _setof1 *msgPtr;   /* used in freeing linked list */

    /*******************************/
    /* Do OSS API initializations. */
    /*******************************/
    ossinit(world, csample); /* initialize OSS run-time environment */
    ossSetEncodingFlags(world, DEBUGPDU); /* Turn on trace capabilities */
    ossSetDecodingFlags(world, DEBUGPDU); 
 
    /*************************************************/
    /* Fill compiler-generated structure with values */
    /*************************************************/
    myPduNum = Msg1_PDU;

    strcpy(myMsg.min, "1234567890");			/* NumericString */

    memcpy(myMsg.memName.value, memNameOctStr, 30);	/* OCTET STRING */
    myMsg.memName.length = 30;

							/* SET OF */
    myMsg.msgs = malloc(sizeof(struct _setof1));	    /* 1st node */

    strcpy(myMsg.msgs->value.text.value, returnMsg1);
    myMsg.msgs->value.text.length = strlen(returnMsg1);    /* VisibleString */
    myMsg.msgs->value.bit_mask = text_present;	     /* OPTIONAL item present*/

    myMsg.msgs->next = malloc(sizeof(struct _setof1));	    /* 2nd node*/

    strcpy(myMsg.msgs->next->value.text.value, returnMsg2);
    myMsg.msgs->next->value.text.length = strlen(returnMsg2);
    myMsg.msgs->next->value.bit_mask = text_present;

    myMsg.msgs->next->next = NULL;           /* terminate SET OF linked list */

    memcpy(myMsg.acntInfo.addr.value, addrOctStr, 30);
    myMsg.acntInfo.addr.length = 30;

    memcpy(myMsg.acntInfo.commt.value, commtOctStr, 30);
    myMsg.acntInfo.commt.length = 30;

    myMsg.memNum = 15;					  /* INTEGER */
    myMsg.bit_mask = memNum_present;

    myMsg.netaddr.networkid = 233;
    myMsg.netaddr.bit_mask = networkid_present;

    myMsg.netaddr.ssn = 215;
    myMsg.netaddr.bit_mask |= ssn_present; /* OR with existing bit_mask */
    myMsg.bit_mask |= netaddr_present;     /* OR with existing bit_mask */

    myMsg.operation.choice = createReqt_chosen;		 /* CHOICE */

    myMsg.operation.u.createReqt.ordId = 5432;

    memcpy(myMsg.operation.u.createReqt.action.value, actionOctStr, 5);
    myMsg.operation.u.createReqt.action.length = 5;

    memcpy(myMsg.operation.u.createReqt.fldNm.value, fldNmOctStr, 15);
    myMsg.operation.u.createReqt.fldNm.length = 15;

    myMsg.operation.u.createReqt.creditChecked = TRUE;	 /* BOOLEAN */

    /* OBJECT IDENTIFIER value is:
    /*             {joint-iso-itu-t(2) internationalRA(23) set(42) 9 12}*/
    myMsg.uniqId.value = NULL;  /* tell ossAsnVal... to allocate memory */
    ossAsnValToEncodedOid(world, "{2 23 42 9 12}", 
        (OssEncodedOID *)&myMsg.uniqId); 

    /* GeneralizedTime value is: "19981106204702Z" or 1998/11/06 20:47:02 GMT */
    myMsg.time.year = 1998;
    myMsg.time.month = 11;
    myMsg.time.day = 6;
    myMsg.time.hour = 20;
    myMsg.time.minute = 47;
    myMsg.time.second = 2; 
    myMsg.time.millisec = 0;
    myMsg.time.mindiff = 0;
    myMsg.time.utc = TRUE;

    /*****************************************************/
    /* Print to stdout the intended input to the encoder */
    /*****************************************************/
    ossPrint(world, "\nThe input to the encoder is:\n\n");
    ossPrintPDU(world, myPduNum, &myMsg);  

    /************************************/
    /* Call the encoder to encode myMsg */
    /************************************/
    encodedData.value = NULL; /* Initialize encoder's output buffer */
    encodedData.length = 0;

    ossPrint(world, "\nCalling the encoder...\n\n");
    /* Encode the data.  Return non-zero for failure.              */
    if (ossEncode(world, myPduNum, &myMsg, &encodedData)) {
        ossPrint(world, "%s\n", ossGetErrMsg(world));  /* print errmsg */
        retcode = 1;
    }
    else
    {
        /*****************************************/
        /* Print the encoded data in hexadecimal */
        /*****************************************/
        ossPrint(world, "\nThe BER encoding is:\n");
        ossPrintHex(world, (char*)encodedData.value, encodedData.length); 

        /************************************/
        /* Call the decoder to decode myMsg */
        /************************************/
	buf = (char *)encodedData.value;    /* Save the address of the buffer
					     * with the encoded data. */

        ossPrint(world, "\nCalling the decoder...\n\n");
        if (ossDecode(world, &myPduNum, &encodedData, (void **)&DecodedDataPtr)) {
            ossPrint(world, "%s\n", ossGetErrMsg(world));  /* print errmsg */
            retcode = 2;
         } else {
            /***********************************************/
            /* Uss ossPrintPDU to print the decoder output */
            /***********************************************/
            ossPrint(world, "\nThe output of the decoder is:\n\n");
            ossPrintPDU(world, myPduNum, DecodedDataPtr);  

            /*****************************************************************/
            /* "Manually" access and print each field in the decoder's output*/
            /*****************************************************************/
            myDecodedDataPrinter(world, DecodedDataPtr);

            ossFreePDU(world, myPduNum, DecodedDataPtr); /* free up PDU */
            retcode = 0;
         }

         /* free the encoder's output buffer*/
         ossFreeBuf(world, buf);
    }

    /**************************************************************************/
    /* Free memory that was allocated when filling compiler-generated structs */
    /**************************************************************************/
    while(myMsg.msgs != NULL)
    {
       msgPtr = myMsg.msgs;
       myMsg.msgs = myMsg.msgs->next;
       free(msgPtr);
    }
    
    free(myMsg.uniqId.value);

    /***************************************/
    /* Release the OSS runtime environment */
    /***************************************/
    ossterm(world);

    return retcode;

} /* end main() */


/***************************************************************/
/* Show how to access the fields found in the decoder's output */
/***************************************************************/
void myDecodedDataPrinter(OssGlobal *world, Msg1 *dptr)
{
   int i;
   struct _setof1 *msgPtr;   /* used in processing linked list */
   OssBuf asnOid = {0, NULL};

   ossPrint(world, "\nPrintout of decoded data:\n");
   ossPrint(world, "Structure:  Msg1\n");
   ossPrint(world, "\tmin:  %s\n", dptr->min);

   ossPrint(world, "\tmemName:  ");
   for(i=0; i < dptr->memName.length; i++)
      ossPrint(world, "%c", dptr->memName.value[i]);

   ossPrint(world, "\n\tReturned Messages:\n");
   for (msgPtr = dptr->msgs; msgPtr != NULL; msgPtr = msgPtr->next)
      if(msgPtr->value.bit_mask & text_present)
         ossPrint(world, "\t\t%s\n", msgPtr->value.text.value);

   ossPrint(world, "\tacntInfo:  \n");
   ossPrint(world, "\t\taddr:  ");
   for(i=0; i<dptr->acntInfo.addr.length; i++)
      ossPrint(world, "%c", dptr->acntInfo.addr.value[i]);

   ossPrint(world, "\n\t\tcommt:  ");
   for(i=0; i<dptr->acntInfo.commt.length; i++)
      ossPrint(world, "%c", dptr->acntInfo.commt.value[i]);

   ossPrint(world, "\n");
   if(dptr->bit_mask & memNum_present)
      ossPrint(world, "\tmemNum:  %d\n", dptr->memNum);

   if(dptr->bit_mask & netaddr_present)
   {
      ossPrint(world, "\tnetaddr:\n");
      if(dptr->netaddr.bit_mask & networkid_present)
         ossPrint(world, "\t\tnetwork id:  %d\n", dptr->netaddr.networkid);
      if(dptr->netaddr.bit_mask & cluster_present)
         ossPrint(world, "\t\tcluster:  %d\n", dptr->netaddr.cluster);
      if(dptr->netaddr.bit_mask & member_present)
         ossPrint(world, "\t\tmember: %d\n", dptr->netaddr.member);
      if(dptr->netaddr.bit_mask & ssn_present)
         ossPrint(world, "\t\tssn: %d\n", dptr->netaddr.ssn);
   }

   ossPrint(world, "\toperation:\n");
   if(dptr->operation.choice == createReqt_chosen)
   {
      ossPrint(world, "\t\tcreateReqt chosen.\n");
      ossPrint(world, "\t\tordId:  %d\n", dptr->operation.u.createReqt.ordId);

      ossPrint(world, "\t\taction:  ");
      for(i=0; i<dptr->operation.u.createReqt.action.length; i++)
         ossPrint(world, "%c", dptr->operation.u.createReqt.action.value[i]);

      ossPrint(world, "\n\t\tfldNm:  ");
      for(i=0; i<dptr->operation.u.createReqt.fldNm.length; i++)
         ossPrint(world, "%c", dptr->operation.u.createReqt.fldNm.value[i]);

      if(dptr->operation.u.createReqt.creditChecked == TRUE)
         ossPrint(world, "\n\t\tcreditChecked:  TRUE");
      else
         ossPrint(world, "\t\tcreditChecked:  FALSE");
   } else if(dptr->operation.choice == updateReqt_chosen) {
      ossPrint(world, "\t\tupdateReqtReqt chosen.\n");
      ossPrint(world, "\t\tordId:  %d\n", dptr->operation.u.updateReqt.ordId);

      ossPrint(world, "\t\taction:  ");
      for(i=0; i<dptr->operation.u.updateReqt.action.length; i++)
         ossPrint(world, "%c", dptr->operation.u.updateReqt.action.value[i]);

      ossPrint(world, "\n\t\toldFldNm:  ");
      for(i=0; i<dptr->operation.u.updateReqt.oldFldNm.length; i++)
         ossPrint(world, "%c",dptr->operation.u.updateReqt.oldFldNm.value[i]);

      ossPrint(world, "\n\t\tnewFldNm:  ");
      for(i=0; i<dptr->operation.u.updateReqt.newFldNm.length; i++)
         ossPrint(world, "%c",dptr->operation.u.updateReqt.newFldNm.value[i]);
   }

   ossEncodedOidToAsnVal(world, (OssEncodedOID *)&dptr->uniqId, &asnOid);
   ossPrint(world, "\n\tuniqId:  %s", asnOid.value);
   world->freep(asnOid.value);
 
   ossPrint(world, "\n\ttime:  %d/%.2d/%.2d %.2d:%.2d:%.2d.%.2d", 
        dptr->time.year, dptr->time.month, dptr->time.day, dptr->time.hour,
        dptr->time.minute, dptr->time.second, dptr->time.millisec);
   if (dptr->time.utc)
	ossPrint(world, " GMT\n");
   else
	ossPrint(world, " %+.2d:%.2d\n", dptr->time.mindiff / 60, 
             dptr->time.mindiff % 60);

   ossPrint(world, "\nFinished printing out decoded data.\n");

} /* end myDecodedDataPrinter() */

